/* 
------------------------------------------------------------
file "hhelp.h" Magic square help Written by Paul Landes on 2/97
Modified:

Copyright (C) 1997 by Paul Landes. This code/program is free 
software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) 
any later version. 

This header file contains object header files and function prototypes
that make it possible to create magic squares of any size.

The object MatrixOdd creates an odd size matrix (2k + 1) with the
De La Loubere algorithm.

The object MatrixSEven creates a matrix of size 4k + 2 by:

Unless otherwise stated at the beginning of each member function,
all functions return standard error status: 1 on fail, 0 on success.

includes:
  iostream.h
  iomanip.h
  matrix.h
------------------------------------------------------------ 
*/

#if !defined (_MHELP_H)
#define _MHELP_H

#if !defined (_IOSTREAM_H)
#include <iostream.h>
#endif

#if !defined (_IOMANIP_H)
#include <iomanip.h>
#endif

#if !defined (_MATRIX_H)

#include "matrix.h"
#endif

//for getting next position in odd size magic squares
void getNextPos(int &, int &, int, int);

typedef Matrix<int> MgPrim;

//---------(abstract)-----------MagicSquare---------------------------
//this is a the abstract base class that creates magic squares.  It
//does this my using another class 'Matrix' for the bookeeping
class MagicSquare
{
public:
  MagicSquare();                       //sets up obligatory data
  virtual ~MagicSquare() { if (mat) delete mat; }

  virtual int makeMagicSquare() = 0;   //creates the magic square
  virtual int init(int) = 0;           //initializes the object

  void printMat() { if ( initialized == true ) printMatrix(*mat); }
  int chkValid();                      //validity device
  int swap(int, int, int, int);        //swap two elements
  void printHtml(bool);
  int getsum() { return magicSum; }
			     

protected:
  int start(int);                      //initializes object
  MgPrim *mat;
  int dim, magicSum;
  //this device is for telling whether the matrix has been created yet
  bool initialized;
};

//------------------------------MagicOdd--------(MagicSquare)---------
//this class creates a magic square of the order 2k + 1 where k is an
//integer.
class MagicOdd : public MagicSquare
{
public:
  MagicOdd():MagicSquare() {}
  ~MagicOdd() {}

  virtual int makeMagicSquare() { return makeOddSquare(1, dim, 0, 0); }
  virtual int init(int value) {
    if ( ( (value + 1) % 2 ) || (value < 3) ) return 1;
    else return start(value);
  }

protected:
  int makeOddSquare(int, int, int, int);  //makes an odd  magic square
};

//------------------------------MagicEven-------(MagicOdd)------------
//this class creates a matrix with a magic square of the order
//4k + 2 where k is an integer.
class MagicSEven : public MagicOdd
{
public:
  MagicSEven():MagicOdd() {}
  ~MagicSEven() {}

  virtual int makeMagicSquare() { return makeSingleEven(); }
  virtual int init(int value) { 
    if ( ( ( value - 2 ) % 4 == 0 ) && (value > 1) )
      return start(value);
    else return 1;
  }

protected:
  int makeSingleEven();                 //makes a 4k + 2 square
};

//------------------------------MagicEven-------(MagicOdd)------------
//This class creates a matrix of doubly even (2k) order.
class MagicDEven : public MagicSquare
{
public:
  MagicDEven():MagicSquare() {}

  virtual int makeMagicSquare() { return makeDEvenSquare(dim); }
  virtual int init(int value) {
    if ( ( value % 4 == 0 ) && (value > 1) )
      return start(value);
    else return 1;
  }

protected:
  void skewExchange(int m, int rowoff, int coloff);
  int makeDEvenSquare(int);
};

//------------------------------MagicControl--------------------------
//this class cordinates all the other Magic classes to create the
//magic square
class MagicControl
{
public:
  MagicControl() { baseMat = 0; }
  ~MagicControl() { if (baseMat) delete baseMat; }

  void printMat() { if (baseMat) baseMat->printMat(); }
  int check() 
  { 
    if (baseMat) return baseMat->chkValid(); 
    else return 0; 
  }
  int init(int n);
  void printHtml(bool bP) { baseMat->printHtml(bP); }
  int getsum() { return baseMat->getsum(); }
       
private:
  MagicSquare *baseMat;
};

#endif
